# -*- coding: utf-8 -*-

import os

__tests__ = os.path.dirname(os.path.realpath(__file__))
__base__ = os.path.realpath(os.path.join(__tests__, '..'))
__resources__ = os.path.join(__tests__, 'resources')



# -*- coding: utf-8 -*-

__author__ = 'Blake Printy'
__email__ = 'bprinty@gmail.com'
__version__ = '0.0.2'

# exposure
from .methods import export
from .methods import render

from .models import line, scatter, bar



import quorra
import pandas

df = pandas.DataFrame({'size': [1,2,3,1,2,3], 'rfu': [1,2,3,5,6,7], 'channel': ['fam', 'fam', 'fam', 'ned', 'ned', 'ned']})

plt = quorra.line()\
    .data(df, x='size', y='rfu', group='channel')\
    .opacity(0.75).lposition("outside")\
    .lshape("circle").labelposition("end")

quorra.export(plt, 'tmp.png')



# -*- coding: utf-8 -*-
#
# Main entry point for quorra
# 
# @author <bprinty@gmail.com>
# ------------------------------------------------


# imports
# -------
import os
import sys
import argparse

from . import __version__


# args
# ----
parser = argparse.ArgumentParser()
# parser.add_argument('-o', '--option', help='Arbitrary option -- uncomment and update to use.', default=None)
subparsers = parser.add_subparsers()


# version
# -------
parser_version = subparsers.add_parser('version')
parser_version.set_defaults(func=lambda x: sys.stderr.write(__version__))


# exec
# ----
def main():
    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# testing for coda
# 
# @author <bprinty@gmail.com>
# ------------------------------------------------


# imporobj
# -------
import unittest
import os
import sys
import subprocess
import pandas
import random
from datetime import datetime

import quorra
from . import __base__


# tests
# -----
class TestEntryPoints(unittest.TestCase):

    def call(self, subcommand, *args):
        return subprocess.check_output('python -m quorra {} {}'.format(
            subcommand, ' '.join(args)
        ), stderr=subprocess.STDOUT, shell=True, cwd=__base__)

    def test_version(self):
        res = self.call('version')
        self.assertTrue(res, quorra.__version__)
        return


class TestExport(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls._time = datetime.now()
        return

    @classmethod
    def tearDownClass(cls):
        qtime = datetime.now() - cls._time
        if qtime.microseconds > 500000:
            sys.stderr.write('\n\nWarning: Export tests took more than 5s to run!\n\n')
        return

    def test_light_export(self):
        data = pandas.DataFrame({
            'x': [i for i in range(0, 10)],
            'y': [round(random.gauss(100, 10), 2) for i in range(0, 10)],
            'group': ['default']*10
        })
        plt = quorra.line()\
            .data(data, x='x', y='y', group='group')\
            .xlabel('X Label').ylabel('Y Label')\
            .zoomable(True)
        outfile = '.quorra-export-test-light-export.png'
        quorra.export(plt, outfile)
        self.assertTrue(os.path.exists(outfile))
        os.remove(outfile)
        return

    def test_heavy_export(self):
        points = 10000
        groups = ['one', 'two', 'three', 'four', 'five']
        data = pandas.DataFrame({
            'x': [i for i in range(0, points)]*len(groups),
            'y': [round(random.gauss(100, 10), 2) for i in range(0, points*len(groups))],
            'group': [grp for grp in groups for j in range(0, points)]
        })
        plt = quorra.line()\
            .data(data, x='x', y='y', group='group')\
            .xlabel('X Label').ylabel('Y Label')
        outfile = '.quorra-export-test-heavy-export.png'
        quorra.export(plt, outfile)
        self.assertTrue(os.path.exists(outfile))
        os.remove(outfile)
        return


# exec
# ----
if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
# Methods for generating quorra plot models.
# 
# @author <bprinty@gmail.com>
# ------------------------------------------------


# imports
# -------
import json
import pandas


# models
# ------
def line():
    return Plot('line')


def density():
    return Plot('density')


def scatter():
    return Plot('scatter')


def bar():
    return Plot('bar')


def histogram():
    return Plot('histogram')


def pie():
    return Plot('pie')


def multiline():
    return Plot('multiline')


# objects
# -------
class Plot(object):

    def __init__(self, model):
        self.model = model
        self.attr = {}
        return

    def __getattr__(self, item):
        def func(value, key=item):
            self.attr[key] = value
            return self
        return func

    def __str__(self):
        ret = 'quorra.{}()'.format(self.model)
        for key in self.attr:
            ret += '\n.{}({})'.format(key, json.dumps(self.attr[key]))
        return ret

    def data(self, data, x=None, y=None, group=None, label=None):
        if isinstance(data, pandas.DataFrame):    
            self.attr['data'] = []
            for idx in range(0, len(data)):
                entry = {}
                if x is not None:
                    entry['x'] = data[x][idx]
                if y is not None:
                    entry['y'] = data[y][idx]
                if group is not None:
                    entry['group'] = data[group][idx]
                if label is not None:
                    entry['label'] = data[label][idx]
                self.attr['data'].append(entry)
        elif isinstance(data, list, tuple):
            self.attr['data'] = data
        else:
            raise AssertionError('Input data must be one of type: (list, tuple, pandas.DataFrame)')
        return self




# -*- coding: utf-8 -*-
#
# Methods for interacting with quorra plots.
# 
# @author <bprinty@gmail.com>
# ------------------------------------------------


# imports
# -------
import os
import uuid
import webbrowser


# config
# ------
_phantom = None
_open = None
_app = None
__src__ = os.path.dirname(os.path.realpath(__file__))
__cwd__ = os.getcwd()
__templates__ = os.path.join(__src__, 'tmpl')


# methods
# -------
def export(plot, filename, width=800, height=600):
    """
    Export plot to file.

    Args:
        plot (quorra.Plot): Quorra plot object to export.
        width (int): Width for plot (pixels).
        height (int): Height for plot (pixels).
        filename (str): Filename to export to.
    """
    global _phantom, __templates__, __cwd__
    if _phantom is None:
        from selenium.webdriver import PhantomJS
        _phantom = PhantomJS(service_log_path=os.path.devnull)
    tmpl = os.path.join(__templates__, 'export.html')
    exp = os.path.join(__cwd__, '.' +  str(uuid.uuid1()) + '.html')
    try:
        with open(tmpl, 'r') as fi, open(exp, 'w') as fo:
            dat = fi.read()
            dat = dat.replace('var plot = undefined;', 'var plot = {};'.format(str(plot)))
            dat = dat.replace('width: 800px;', 'width: {}px;'.format(width))
            dat = dat.replace('height: 500px;', 'height: {}px;'.format(height))
            fo.write(dat)
        _phantom.get('file://' + exp)
        _phantom.save_screenshot(filename.replace('.png', '') + '.png')
    finally:
        os.remove(exp)
    return


def render(plot, width=800, height=600, append=False):
    """
    Update current view with new plot.

    Args:
        plot (quorra.Plot): Quorra plot object to render.
        width (int): Width for plot (pixels).
        height (int): Height for plot (pixels).
        append (bool): Whether or not to append the plot
            to the current view.
    """
    global _open, __templates__
    if _open is None:
        # this should change later to an actual server that
        # waits for data and can support multiple plots at
        # the same time
        _open = os.path.join('/tmp/quorra-server.html')
    tmpl = os.path.join(__templates__, 'render.html')
    with open(tmpl, 'r') as fi, open(_open, 'w') as fo:
        dat = fi.read()
        dat = dat.replace('var plot = undefined;', 'var plot = {};'.format(str(plot)))
        dat = dat.replace('width: 800px;', 'width: {}px;'.format(width))
        dat = dat.replace('height: 500px;', 'height: {}px;'.format(height))
        fo.write(dat)
    webbrowser.open(_open)
    return




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# package setup
# 
# @author <bprinty@gmail.com>
# ------------------------------------------------


# config
# ------
import quorra
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


# requirements
# ------------
with open('requirements.txt', 'r') as reqs:
    requirements = reqs.readlines()

test_requirements = [
    'nose',
    'nose-parameterized'
]


# files
# -----
with open('README.rst') as fi:
    readme = fi.read()


# exec
# ----
setup(
    name='quorra',
    version=quorra.__version__,
    description='A python extension for generating quorra.js plots directly from python',
    long_description=readme,
    author='Blake Printy',
    author_email='bprinty@gmail.com',
    url='https://github.com/bprinty/quorra',
    packages=['quorra'],
    package_data={'quorra': ['tmpl/*.html']},
    package_dir={'quorra': 'quorra'},
    # entry_points={
    #     'console_scripts': [
    #         'quorra = quorra.__main__:main'
    #     ]
    # },
    include_package_data=True,
    install_requires=requirements,
    license='Apache-2.0',
    zip_safe=False,
    keywords=['quorra', 'visualization', 'plotting', 'charts', 'data'],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apple Public Source License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='nose.collector',
    tests_require=test_requirements
)
